`include "macros.vh"

module CPU(
	input clk, 
	input reset,
	input [15:0] dataRom,
	output reg read, 
	output reg readIO, 
	output reg powerLed,
	output reg statusLed,
	output reg errorLed,
	output reg [6:0] addressRam,
	output [9:0] addressRom,
	inout [7:0] dataRam
	);

	//private registers 
	reg state;
	reg [3:0] i;
	reg [7:0] R, Rd, Rr;
	reg [9:0] programCounter;
	reg [7:0] temp [1:0];
	
	//public registers
	reg [7:0] localReg [15:0];
	reg [7:0] flags;
	reg [7:0] dataRamValue;
	reg [6:0] addressRamValue;	
		
	assign addressRom = programCounter;
	assign dataRam = read? 8'bz : dataRamValue;
	
	task setFlagsMath;
		begin
			flags[`H] = Rd[3] * Rr[3] + Rr[3] * !R[3] + !R[3] * Rd[3];
			// TODO: complete it.
		end
	endtask

	task displayDebug;
		begin
			//PC
			$display("PC = %d, instruction = %h\n", programCounter, dataRom);
					
			//registers
			for(i=0; i<15; i=i+1) 
			begin
				$write("R%2d = %h  |  ", i + 16, localReg[i]);
			end
				$display("R%2d = %h\n", i + 16, localReg[i]);
		
			//flags
			$display("I = %d | T = %d | H = %d | S = %d | V = %d | N = %d | Z = %d | C = %d\n", 
				flags[7], flags[6], flags[5], flags[4], flags[3], flags[2], flags[1], flags[0]);
		end
	endtask	
			
	
	always @(posedge clk, posedge reset)
	begin
		statusLed = 0;
		powerLed = 1;
			
		if(reset)  
		begin
			$display("resetting...");
			programCounter = 8'h00;
			state = `EXECUTE;
			flags = 8'h00;			
			read = 1;
			
			statusLed = 0;
			errorLed = 0;
		end else begin
			displayDebug;
			read = 1;
			case(state) 
				`MEM_READ: begin
					//for LDS memory access operation
					if(dataRom[15:11] == 5'b10100) begin
						localReg[dataRom[7:4]] = dataRam;
					end
					programCounter = programCounter + 1;
					state = `EXECUTE;
				end //MEM_READ
				
				`EXECUTE: begin
					//16 bits
					case(dataRom[15:0])
						16'b000000000000: begin
							//NOP
							$display("NOP");
						end

						default: begin
							//8 bits
							case(dataRom[15:8])
								8'b00001111: begin
									//ADD
									$display("ADD R%2d, R%2d", dataRom[7:4] + 16, dataRom[3:0] + 16);
									setFlagsMath();
								end
								
								8'b00011111: begin
									//ADC
									$display("ADC R%2d, R%2d", dataRom[7:4] + 16, dataRom[3:0] + 16);
									// TODO:
								end
								
								default: begin
								  // 7 bits
								  case (dataRom[15:9])
								    7'b1001010: begin // NEG
								    end
								    default: begin 
								      // 6 bits:
								      case (dataRom[15:10])  
								        6'b00100: begin //AND
								        end
								        6'b001010: begin // OR
								        end
								        6'b001001: begin // EOR
								        end
								        default: begin
								          // 4 bits:
								          case (dataRom[15:12])  
								            4'b0101: begin // SUB
								            end
								          endcase
								        end
								      endcase // 6 bits
								    end
								  endcase // 7 bis
								end	
							endcase //8 bits
						end
					endcase //16 bits
					
					programCounter = programCounter + 1;
				end //EXECUTE
			endcase //case(state)		
		end //if(reset)
	end //always @(posedge clk, posedge reset)  
endmodule
