
module ROM(
	input clk,
	input [9:0] addressRom,
	output reg [15:0] dataRom
	);
	
	reg [15:0] rom [0:1023];
	
	reg [7:0] length, type, crc, checkCrc;
	reg [15:0] dataAddress;
	reg [19:0] offset;
	
	reg [7:0] char; 
	reg [31:0] dataFile;
	integer i;
	
	
	//convert a character to hex value
	function [3:0] char2nibble;
		input[7:0] char;
		begin
			if(char >= "0" && char <="9") 
			begin 
				char2nibble = char - "0";
			end else begin
				char2nibble = char - "A" + 10;	
			end				
		end
	endfunction
	
	//convert two characters to a byte
	function [7:0] char2byte;
		input [7:0] char1, char2;
		begin
			char2byte = {char2nibble(char1), char2nibble(char2)};
		end
	endfunction

	//read two characters from file and convert them to one byte
	function [7:0] readByte;
		input [31:0] file;
		integer char1, char2;
		begin
			char1 = $fgetc(file);
			char2 = $fgetc(file);
			checkCrc = checkCrc + char2byte(char1, char2);
			readByte = char2byte(char1, char2);
		end
	endfunction
	
	//zero-initialize the memory
	task resetMem;
		begin
			for (i=0; i<1024; i=i+1)
			begin
				rom[i] = 0;
			end
		end
	endtask
	
	
	initial
	begin
		resetMem;
		
		dataFile = $fopen("IO.hex", "r");
		checkCrc = 0;		
		offset = 0;
		
		begin: loadHex
			while(!$feof(dataFile))
			begin
				$fgetc(dataFile); //':'
				length = readByte(dataFile); 
				dataAddress[15:8] = readByte(dataFile); 
				dataAddress[7:0] = readByte(dataFile); 
				type = readByte(dataFile);
				 
				case(type)
					8'h00:
					begin
						for(i=0; i<length; i=i+2) 
						begin
							rom[(dataAddress + offset + i) / 2][7:0] = readByte(dataFile);
							rom[(dataAddress + offset + i) / 2][15:8] = readByte(dataFile);
						end
					end
					
					8'h02:
					begin 
						offset[19:12] = readByte(dataFile);
						offset[11:4] = readByte(dataFile);
					end
				endcase
				 
				crc = readByte(dataFile);
				if(checkCrc != 00) 
				begin
					resetMem;
					$display("ROM: crc error");
					$finish;
				end else begin
					if(type == 8'h01)
					begin
						disable loadHex;
					end
				end
				
				$fgetc(dataFile); //'\n'
			end //while
		end //loadHex
	end //initial
	
	always @(negedge clk) 
	begin
		dataRom = rom[addressRom];
	end //always @(negedge clk)
endmodule
